#version 130
#extension GL_EXT_gpu_shader4 : enable
// the version and open GL extension
// should be the first line of the shader
/////////////////////////////////////////////////////////////////////////////////
// Mandelbrot - GavinMod01.fsh by   whereisgavin
//https://www.shadertoy.com/view/....
//Licence : Creative Commons Attribution-ShareAlike 4.0
//http://creativecommons.org/licences/by-sa/4.0
// Adapted, trivialy, for use in VGHD player
/////////////////////////////////////////////
uniform float u_Elapsed;    // The elapsed time in seconds
uniform vec2  u_WindowSize; // Window dimensions in pixels

#define iTime u_Elapsed*0.314159  //*0.1666
#define iResolution u_WindowSize

//#define mouse AUTO_MOUSE
//#define MOUSE_SPEED vec2(vec2(0.5,0.577777) * 0.25)
//#define MOUSE_POS   vec2((1.0+cos(iTime*MOUSE_SPEED))*u_WindowSize/2.0)
//#define MOUSE_PRESS vec2(0.0,0.0)
//#define AUTO_MOUSE  vec4( MOUSE_POS, MOUSE_PRESS )
//#define RIGID_SCROLL
// alternatively use static mouse definition
#define iMouse vec4(0.0,0.0, 0.0,0.0)
//#define iMouse vec4(512,256,180,120)
uniform sampler2D iChannel0;
uniform sampler2D iChannel1;
uniform sampler2D iChannel2;
uniform sampler2D iChannel3;
vec4 texture2D_Fract(sampler2D sampler,vec2 P) {return texture2D(sampler,fract(P));}
vec4 texture2D_Fract(sampler2D sampler,vec2 P, float Bias) {return texture2D(sampler,fract(P),Bias);}
#define texture2D texture2D_Fract

vec2 cMult(vec2 z1, vec2 z2)
{
    return mat2( z1, -z1.y, z1.x ) * z2;
}

vec2 F(vec2 z, vec2 c)
{
    return cMult(z, z) + c;
}

//void mainImage( out vec4 fragColor, in vec2 fragCoord )
///////////////////////////////////////////////////////////////////////////////// 
// need to convert this from a void to a function and call it by adding
// a void main(void) { to the end of the shader
// what type of variable will the function return?, it is a color and needs to be a vec4
// change void to vec4 
//void MainImage(out vec4 fragColor, in vec2 fragCoord) 
vec4 mainImage( out vec4 fragColor, in vec2 fragCoord )
{
    // Normalized pixel coordinates (from 0 to 1)
    //vec2 uv = fragCoord/iResolution.xy;
    //uv = (uv - 0.5) * 2.0;
    //float aspectRatio = iResolution.y / iResolution.x;
    //uv.x = uv.x / aspectRatio;
    
    //Thanks FabriceNeyret2 for quicker UV!
    vec2 uv = ( 2.* fragCoord - iResolution.xy ) / iResolution.y;
    
    //Zooming
    float speed = 20.0;
    float zoom = iTime / 10.0f;
    zoom =  0.5 + zoom * zoom * speed;
    
    //Center Point
    vec2 point = vec2(-0.77568377, 0.13646737);
    
    //Transform
    vec3 transform = vec3(point, zoom);
    uv /= transform.z;
    uv += transform.xy;
    
    //Calculate F(z) = z^2 + c
    float maxIterations = 200.0;
    float numIterations = 0.0;
    float threshold = 400.0;
    
    vec2 z = vec2(0.0, 0.0);
    vec2 c = vec2(uv.x, uv.y);
    
    float magnitude = 0.0;
    while (magnitude < threshold)
    {
        z = F(z, c);
        numIterations = numIterations + 1.0;
        if (numIterations > maxIterations) break;
        magnitude = length(z);
    }
    
    float value = 1.0 - (numIterations / maxIterations);
    
    //Color
    //vec3 col = vec3(value); // grayscale
    vec3 col = vec3(0.9 * value, 1.2 * value * value, value); // soft purple
    
    // Output to screen
    fragColor = vec4(col,1.0);
/////////////////////////////////////////////////////////////////////////////////
//the function needs to return a value. 
//it needs to be a vec4
//we will return the varable fragColor 
// usual place for fragColor = vec4( color, 1.0 ); bring the } down below
return fragColor; 
}

///////////////////////////////////////////////////////////////////////////////// 
void main(void) { // this will be run for every pixel of gl_FragCoord.xy
vec4 fragColor = vec4(1.0); // initialize variable fragColor as a vec4 
vec4 cc = mainImage(fragColor, gl_FragCoord.xy); // call function mainImage and assign the return vec4 to cc
gl_FragColor = vec4(cc) * gl_Color; // set the pixel to the value of vec4 cc  and..
}

// ..uses the values of any Color: or Opacity:
// clauses (and any Animate clauses applied to these properties) 
// appearing in the Sprite, Quad or other node invoking the shader 
// in the .scn file.

